from django.conf import settings
from django.utils.module_loading import import_string

SIMPLE_PERMS_MODULE_NAME = getattr(settings, "SIMPLE_PERMS_MODULE_NAME", "perms")
SIMPLE_PERMS_GLOBAL_DEFAULT_PERMISSION = getattr(
    settings, "SIMPLE_PERMS_GLOBAL_DEFAULT_PERMISSION", None
)
if SIMPLE_PERMS_GLOBAL_DEFAULT_PERMISSION:
    SIMPLE_PERMS_GLOBAL_DEFAULT_PERMISSION = import_string(
        SIMPLE_PERMS_GLOBAL_DEFAULT_PERMISSION
    )
else:
    # pylint: disable-next=unnecessary-lambda-assignment
    SIMPLE_PERMS_GLOBAL_DEFAULT_PERMISSION = lambda user, obj, perm: False



_registry = {}


def register(app_label, logic_class):
    if app_label in _registry:
        raise AttributeError(
            f'The "{app_label}" app is already registerd by simple_perms'
        )
    _registry[app_label] = logic_class()


def get_app_logic(app_label):
    return _registry.get(app_label, None)



from django.apps import AppConfig
from django.utils.module_loading import autodiscover_modules

from .settings import SIMPLE_PERMS_MODULE_NAME


class SimplePermsConfig(AppConfig):
    name = "simple_perms"
    verbose_name = "Simple perms"

    def ready(self):
        autodiscover_modules(SIMPLE_PERMS_MODULE_NAME)



from .registry import get_app_logic
from .settings import SIMPLE_PERMS_GLOBAL_DEFAULT_PERMISSION


class PermissionBackend:
    def has_perm(self, user, perm, obj=None):
        try:
            app_label, perm_name = perm.split(".")
        except Exception as e:
            raise AttributeError(
                f'The given perm attribute "{perm}" hasn\'t the required format : '
                '"app_label.permission_name"'
            ) from e

        logic = get_app_logic(app_label)

        if logic:
            if hasattr(logic, perm_name):
                return getattr(logic, perm_name)(user, obj, perm)

            return logic.default_permission(user, obj, perm)

        return SIMPLE_PERMS_GLOBAL_DEFAULT_PERMISSION(user, obj, perm)

    def authenticate(self, *args):
        return None



from .settings import SIMPLE_PERMS_GLOBAL_DEFAULT_PERMISSION


class PermissionLogic:
    def default_permission(self, user, obj, perm):
        return SIMPLE_PERMS_GLOBAL_DEFAULT_PERMISSION(user, obj, perm)



# -*- coding: utf-8 -*-
# pylint: disable=line-too-long


class AssertPermissions:
    """
    Class for testing permissions

    :Example:

    >>> permissions = [
    >>>     { 'usr': 'admin', 'perm': 'contracts.can_use_contracts_ui', 'args': (None,),           'result': True, },
    >>>     { 'usr': 'admin', 'perm': 'contracts.add',                  'args': (None,),           'result': True, },
    >>>     { 'usr': 'admin', 'perm': 'contracts.view',                 'args': (self.contract, ), 'result': True, },
    >>>     { 'usr': 'admin', 'perm': 'contracts.change',               'args': (self.contract, ), 'result': True, },
    >>> ]
    >>> self.assertPerms(permissions)

    Where:
    usr is a class attribute of self.
    """

    __exceptions__ = []

    def assertPerms(self, permissions):
        """
        Test if permissions are correct given to the dict of permissions
        """
        self.__exceptions__ = []

        for permission in permissions:
            self._test_permission_(permission)

        if self.__exceptions__:
            e = self.__exceptions__[0]
            for ex in self.__exceptions__[1:]:
                e.args = e.args + ex.args

            raise e

    def _test_permission_(self, permission):
        """
        Test if permission is correct.
        Raise AssertionError if result is not correct.

        >>> ======================================================================
        >>> FAIL: test_permissions_of_team_leader (contracts.tests.perms.TestContractPermission)
        >>> ----------------------------------------------------------------------
        >>> Traceback (most recent call last):
        >>>   File "/app/django/contracts/tests/perms.py", line 138, in test_permissions_of_team_leader
        >>>     self.assertPerms(permissions)
        >>>   File "/app/django/contracts/tests/perms.py", line 18, in assertPerms
        >>>     self._test_permission_(permission)
        >>>   File "/app/django/contracts/tests/perms.py", line 36, in _test_permission_
        >>>     raise e
        >>>   File "/app/django/contracts/tests/perms.py", line 31, in _test_permission_
        >>>     getattr(self, permission['usr']).has_perm(permission['perm'], *permission['args'])
        >>> AssertionError: ('False is not true', 'PERM ERROR user:team_leader perm:contracts.view - attended: True')

        """
        if permission["result"]:
            action = self.assertTrue  # pylint: disable=no-member
        else:
            action = self.assertFalse  # pylint: disable=no-member

        try:
            action(
                getattr(self, permission["usr"]).has_perm(
                    permission["perm"], *permission["args"]
                )
            )
        except Exception as e:  # pylint: disable=broad-except
            my_err = f"PERM ERROR user:{permission['usr']} perm:{permission['perm']} args:{permission['args']}: "
            e.args = (f"{my_err}",)
            self.__exceptions__.append(e)
